@@ -205,8 +205,8 @@ void      gst_tag_merge_strings_with_comma (GValue * dest, const GValue * src);
 /* basic tag support */
 gboolean               gst_tag_exists          (const gchar * tag);
 GType                  gst_tag_get_type        (const gchar * tag);
-G_CONST_RETURN gchar * gst_tag_get_nick        (const gchar * tag);
-G_CONST_RETURN gchar * gst_tag_get_description (const gchar * tag);
+const gchar *          gst_tag_get_nick        (const gchar * tag);
+const gchar *          gst_tag_get_description (const gchar * tag);
 GstTagFlag             gst_tag_get_flag        (const gchar * tag);
 gboolean               gst_tag_is_fixed        (const gchar * tag);
 
@@ -253,7 +253,7 @@ void         gst_tag_list_foreach           (const GstTagList * list,
                                              GstTagForeachFunc  func,
                                              gpointer           user_data);
 
-G_CONST_RETURN GValue *
+const GValue *
              gst_tag_list_get_value_index   (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index);
@@ -579,6 +579,19 @@ gboolean     gst_tag_list_get_buffer_index  (const GstTagList * list,
  * Since: 0.10.14
  */
 #define GST_TAG_COPYRIGHT_URI          "copyright-uri"
+/**
+ * GST_TAG_ENCODED_BY:
+ *
+ * name of the person or organisation that encoded the file. May contain a
+ * copyright message if the person or organisation also holds the copyright
+ * (string)
+ *
+ * Note: do not use this field to describe the encoding application. Use
+ * #GST_TAG_APPLICATION_NAME or #GST_TAG_COMMENT for that.
+ *
+ * Since: 0.10.33
+ */
+#define GST_TAG_ENCODED_BY             "encoded-by"
 /**
  * GST_TAG_CONTACT:
  *
@@ -654,7 +667,8 @@ gboolean     gst_tag_list_get_buffer_index  (const GstTagList * list,
 /**
  * GST_TAG_NOMINAL_BITRATE:
  *
- * nominal bitrate in bits/s (unsigned integer)
+ * nominal bitrate in bits/s (unsigned integer). The actual bitrate might be
+ * different from this target bitrate.
  */
 #define GST_TAG_NOMINAL_BITRATE        "nominal-bitrate"
 /**
