@@ -1,6 +1,7 @@
 /* GStreamer
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *                    2000 Wim Taymans <wtay@chello.be>
+ * Copyright (C) 2011 Tim-Philipp Müller <tim centricular net>
  *
  * gsturi.c: register URI handlers
  *
@@ -51,6 +52,14 @@
 GST_DEBUG_CATEGORY_STATIC (gst_uri_handler_debug);
 #define GST_CAT_DEFAULT gst_uri_handler_debug
 
+enum
+{
+  NEW_URI,
+  LAST_SIGNAL
+};
+
+static guint gst_uri_handler_signals[LAST_SIGNAL] = { 0 };
+
 static void gst_uri_handler_base_init (gpointer g_class);
 
 GType
@@ -93,12 +102,13 @@ gst_uri_handler_base_init (gpointer g_class)
     /**
      * GstURIHandler::new-uri:
      * @handler: The #GstURIHandler which emitted the signal
-     * @uri: The new URI, or NULL if the URI was removed
+     * @uri: (transfer none): The new URI, or NULL if the URI was removed
      *
      * The URI of the given @handler has changed.
      */
 
-    g_signal_new ("new-uri", GST_TYPE_URI_HANDLER, G_SIGNAL_RUN_LAST,
+    gst_uri_handler_signals[NEW_URI] =
+        g_signal_new ("new-uri", GST_TYPE_URI_HANDLER, G_SIGNAL_RUN_LAST,
         G_STRUCT_OFFSET (GstURIHandlerInterface, new_uri), NULL, NULL,
         gst_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
     initialized = TRUE;
@@ -199,8 +209,7 @@ escape_string_internal (const gchar * string, UnsafeCharacterSet mask)
   return result;
 }
 
-/**
- * escape_string:
+/* escape_string:
  * @string: string to be escaped
  *
  * Escapes @string, replacing any and all special characters
@@ -242,8 +251,7 @@ unescape_character (const char *scanner)
   return (first_digit << 4) | second_digit;
 }
 
-/**
- * unescape_string:
+/* unescape_string:
  * @escaped_string: an escaped URI, path, or other string
  * @illegal_characters: a string containing a sequence of characters
  * considered "illegal", '\0' is automatically in this list.
@@ -379,7 +387,7 @@ gst_uri_get_protocol (const gchar * uri)
 
 /**
  * gst_uri_has_protocol:
- * @uri: an URI string
+ * @uri: a URI string
  * @protocol: a protocol string (e.g. "http")
  *
  * Checks if the protocol of a given valid URI matches @protocol.
@@ -414,8 +422,11 @@ gst_uri_has_protocol (const gchar * uri, const gchar * protocol)
  * the hostname if one is specified. The returned string must be freed using
  * g_free().
  *
- * Returns: The location for this URI. Returns NULL if the URI isn't valid. If
- * the URI does not contain a location, an empty string is returned.
+ * Free-function: g_free
+ *
+ * Returns: (transfer full) (array zero-terminated=1): the location for this
+ *     URI. Returns NULL if the URI isn't valid. If the URI does not contain
+ *     a location, an empty string is returned.
  */
 gchar *
 gst_uri_get_location (const gchar * uri)
@@ -447,19 +458,22 @@ gst_uri_get_location (const gchar * uri)
 #endif
 
   GST_LOG ("extracted location '%s' from URI '%s'", GST_STR_NULL (unescaped),
-      uri);;
+      uri);
   return unescaped;
 }
 
 /**
  * gst_uri_construct:
  * @protocol: Protocol for URI
- * @location: Location for URI
+ * @location: (array zero-terminated=1) (transfer none): Location for URI
  *
  * Constructs a URI for a given valid protocol and location.
  *
- * Returns: a new string for this URI. Returns NULL if the given URI protocol
- * is not valid, or the given location is NULL.
+ * Free-function: g_free
+ *
+ * Returns: (transfer full) (array zero-terminated=1): a new string for this
+ *     URI. Returns NULL if the given URI protocol is not valid, or the given
+ *     location is NULL.
  */
 gchar *
 gst_uri_construct (const gchar * protocol, const gchar * location)
@@ -495,9 +509,9 @@ search_by_entry (GstPluginFeature * feature, gpointer search_entry)
 
   if (!GST_IS_ELEMENT_FACTORY (feature))
     return FALSE;
-  factory = GST_ELEMENT_FACTORY (feature);
+  factory = GST_ELEMENT_FACTORY_CAST (feature);
 
-  if (gst_element_factory_get_uri_type (factory) != entry->type)
+  if (factory->uri_type != entry->type)
     return FALSE;
 
   protocols = gst_element_factory_get_uri_protocols (factory);
@@ -517,11 +531,8 @@ search_by_entry (GstPluginFeature * feature, gpointer search_entry)
 }
 
 static gint
-sort_by_rank (gconstpointer a, gconstpointer b)
+sort_by_rank (GstPluginFeature * first, GstPluginFeature * second)
 {
-  GstPluginFeature *first = GST_PLUGIN_FEATURE (a);
-  GstPluginFeature *second = GST_PLUGIN_FEATURE (b);
-
   return gst_plugin_feature_get_rank (second) -
       gst_plugin_feature_get_rank (first);
 }
@@ -576,11 +587,11 @@ gst_uri_protocol_is_supported (const GstURIType type, const gchar * protocol)
  * gst_element_make_from_uri:
  * @type: Whether to create a source or a sink
  * @uri: URI to create an element for
- * @elementname: Name of created element, can be NULL.
+ * @elementname: (allow-none): Name of created element, can be NULL.
  *
  * Creates an element for handling the given URI.
  *
- * Returns: a new element or NULL if none could be created
+ * Returns: (transfer full): a new element or NULL if none could be created
  */
 GstElement *
 gst_element_make_from_uri (const GstURIType type, const gchar * uri,
@@ -603,10 +614,11 @@ gst_element_make_from_uri (const GstURIType type, const gchar * uri,
     return NULL;
   }
 
-  possibilities = g_list_sort (possibilities, sort_by_rank);
+  possibilities = g_list_sort (possibilities, (GCompareFunc) sort_by_rank);
   walk = possibilities;
   while (walk) {
-    if ((ret = gst_element_factory_create (GST_ELEMENT_FACTORY (walk->data),
+    if ((ret =
+            gst_element_factory_create (GST_ELEMENT_FACTORY_CAST (walk->data),
                 elementname)) != NULL) {
       GstURIHandler *handler = GST_URI_HANDLER (ret);
 
@@ -643,8 +655,13 @@ gst_uri_handler_get_uri_type (GstURIHandler * handler)
 
   iface = GST_URI_HANDLER_GET_INTERFACE (handler);
   g_return_val_if_fail (iface != NULL, GST_URI_UNKNOWN);
-  g_return_val_if_fail (iface->get_type != NULL, GST_URI_UNKNOWN);
-  ret = iface->get_type ();
+  g_return_val_if_fail (iface->get_type != NULL
+      || iface->get_type_full != NULL, GST_URI_UNKNOWN);
+
+  if (iface->get_type != NULL)
+    ret = iface->get_type ();
+  else
+    ret = iface->get_type_full (G_OBJECT_TYPE (handler));
   g_return_val_if_fail (GST_URI_TYPE_IS_VALID (ret), GST_URI_UNKNOWN);
 
   return ret;
@@ -657,9 +674,9 @@ gst_uri_handler_get_uri_type (GstURIHandler * handler)
  * Gets the list of protocols supported by @handler. This list may not be
  * modified.
  *
- * Returns: the supported protocols.
- * Returns NULL if the @handler isn't implemented properly, or the @handler
- * doesn't support any protocols.
+ * Returns: (transfer none) (array zero-terminated=1) (element-type utf8): the
+ *     supported protocols. Returns NULL if the @handler isn't implemented
+ *     properly, or the @handler doesn't support any protocols.
  */
 gchar **
 gst_uri_handler_get_protocols (GstURIHandler * handler)
@@ -690,11 +707,11 @@ gst_uri_handler_get_protocols (GstURIHandler * handler)
  *
  * Gets the currently handled URI.
  *
- * Returns: the URI currently handled by the @handler.
- * Returns NULL if there are no URI currently handled. The returned
- * string must not be modified or freed.
+ * Returns: (transfer none): the URI currently handled by the @handler.
+ *   Returns NULL if there are no URI currently handled. The
+ *   returned string must not be modified or freed.
  */
-G_CONST_RETURN gchar *
+const gchar *
 gst_uri_handler_get_uri (GstURIHandler * handler)
 {
   GstURIHandlerInterface *iface;
@@ -764,5 +781,117 @@ gst_uri_handler_new_uri (GstURIHandler * handler, const gchar * uri)
 {
   g_return_if_fail (GST_IS_URI_HANDLER (handler));
 
-  g_signal_emit_by_name (handler, "new-uri", uri);
+  g_signal_emit (handler, gst_uri_handler_signals[NEW_URI], 0, uri);
+}
+
+static gchar *
+gst_file_utils_canonicalise_path (const gchar * path)
+{
+  gchar **parts, **p, *clean_path;
+
+#ifdef G_OS_WIN32
+  {
+    GST_WARNING ("FIXME: canonicalise win32 path");
+    return g_strdup (path);
+  }
+#endif
+
+  parts = g_strsplit (path, "/", -1);
+
+  p = parts;
+  while (*p != NULL) {
+    if (strcmp (*p, ".") == 0) {
+      /* just move all following parts on top of this, incl. NUL terminator */
+      g_free (*p);
+      g_memmove (p, p + 1, (g_strv_length (p + 1) + 1) * sizeof (gchar *));
+      /* re-check the new current part again in the next iteration */
+      continue;
+    } else if (strcmp (*p, "..") == 0 && p > parts) {
+      /* just move all following parts on top of the previous part, incl.
+       * NUL terminator */
+      g_free (*(p - 1));
+      g_free (*p);
+      g_memmove (p - 1, p + 1, (g_strv_length (p + 1) + 1) * sizeof (gchar *));
+      /* re-check the new current part again in the next iteration */
+      --p;
+      continue;
+    }
+    ++p;
+  }
+  if (*path == '/') {
+    guint num_parts;
+
+    num_parts = g_strv_length (parts) + 1;      /* incl. terminator */
+    parts = g_renew (gchar *, parts, num_parts + 1);
+    g_memmove (parts + 1, parts, num_parts * sizeof (gchar *));
+    parts[0] = g_strdup ("/");
+  }
+
+  clean_path = g_build_filenamev (parts);
+  g_strfreev (parts);
+  return clean_path;
+}
+
+static gboolean
+file_path_contains_relatives (const gchar * path)
+{
+  return (strstr (path, "/./") != NULL || strstr (path, "/../") != NULL ||
+      strstr (path, G_DIR_SEPARATOR_S "." G_DIR_SEPARATOR_S) != NULL ||
+      strstr (path, G_DIR_SEPARATOR_S ".." G_DIR_SEPARATOR_S) != NULL);
+}
+
+/**
+ * gst_filename_to_uri:
+ * @filename: absolute or relative file name path
+ * @error: pointer to error, or NULL
+ *
+ * Similar to g_filename_to_uri(), but attempts to handle relative file paths
+ * as well. Before converting @filename into an URI, it will be prefixed by
+ * the current working directory if it is a relative path, and then the path
+ * will be canonicalised so that it doesn't contain any './' or '../' segments.
+ *
+ * On Windows #filename should be in UTF-8 encoding.
+ *
+ * Since: 0.10.33
+ */
+gchar *
+gst_filename_to_uri (const gchar * filename, GError ** error)
+{
+  gchar *abs_location = NULL;
+  gchar *uri, *abs_clean;
+
+  g_return_val_if_fail (filename != NULL, NULL);
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+  if (g_path_is_absolute (filename)) {
+    if (!file_path_contains_relatives (filename)) {
+      uri = g_filename_to_uri (filename, NULL, error);
+      goto beach;
+    }
+
+    abs_location = g_strdup (filename);
+  } else {
+    gchar *cwd;
+
+    cwd = g_get_current_dir ();
+    abs_location = g_build_filename (cwd, filename, NULL);
+    g_free (cwd);
+
+    if (!file_path_contains_relatives (abs_location)) {
+      uri = g_filename_to_uri (abs_location, NULL, error);
+      goto beach;
+    }
+  }
+
+  /* path is now absolute, but contains '.' or '..' */
+  abs_clean = gst_file_utils_canonicalise_path (abs_location);
+  GST_LOG ("'%s' -> '%s' -> '%s'", filename, abs_location, abs_clean);
+  uri = g_filename_to_uri (abs_clean, NULL, error);
+  g_free (abs_clean);
+
+beach:
+
+  g_free (abs_location);
+  GST_DEBUG ("'%s' -> '%s'", filename, uri);
+  return uri;
 }
