@@ -248,6 +248,27 @@ struct _GstBaseTransformPrivate
   gboolean proxy_alloc;
   GstCaps *sink_alloc;
   GstCaps *src_alloc;
+
+  /*
+   * This flag controls if basetransform should explicitly
+   * do a pad alloc when it receives a buffer even if it operates on
+   * passthrough, this is needed to check for downstream caps suggestions
+   * and this newly alloc'ed buffer is discarded.
+   *
+   * Without this flag basetransform would try a pad alloc whenever it
+   * gets a new buffer and pipelines like:
+   * "src ! basetrans1 ! basetrans2 ! basetrans3 ! sink"
+   * Would have a 3 pad allocs for each buffer pushed downstream from the src.
+   *
+   * This flag is set to TRUE on start up, on setcaps and when a buffer is
+   * pushed downstream. It is set to FALSE after a pad alloc has been requested
+   * downstream.
+   * The rationale is that when a pad alloc flows through the pipeline, all
+   * basetransform elements on passthrough will avoid pad alloc'ing when they
+   * get the buffer.
+   */
+  gboolean force_alloc;
+
   /* upstream caps and size suggestions */
   GstCaps *sink_suggest;
   guint size_suggest;
@@ -258,6 +279,8 @@ struct _GstBaseTransformPrivate
   /* QoS stats */
   guint64 processed;
   guint64 dropped;
+
+  GstClockTime last_stop_out;
 };
 
 static GstElementClass *parent_class = NULL;
@@ -321,9 +344,13 @@ static GstFlowReturn gst_base_transform_chain (GstPad * pad,
     GstBuffer * buffer);
 static GstCaps *gst_base_transform_getcaps (GstPad * pad);
 static gboolean gst_base_transform_acceptcaps (GstPad * pad, GstCaps * caps);
+static gboolean gst_base_transform_acceptcaps_default (GstBaseTransform * trans,
+    GstPadDirection direction, GstCaps * caps);
 static gboolean gst_base_transform_setcaps (GstPad * pad, GstCaps * caps);
 static GstFlowReturn gst_base_transform_buffer_alloc (GstPad * pad,
     guint64 offset, guint size, GstCaps * caps, GstBuffer ** buf);
+static gboolean gst_base_transform_query (GstPad * pad, GstQuery * query);
+static const GstQueryType *gst_base_transform_query_type (GstPad * pad);
 
 /* static guint gst_base_transform_signals[LAST_SIGNAL] = { 0 }; */
 
@@ -368,6 +395,8 @@ gst_base_transform_class_init (GstBaseTransformClass * klass)
   klass->passthrough_on_same_caps = FALSE;
   klass->event = GST_DEBUG_FUNCPTR (gst_base_transform_sink_eventfunc);
   klass->src_event = GST_DEBUG_FUNCPTR (gst_base_transform_src_eventfunc);
+  klass->accept_caps =
+      GST_DEBUG_FUNCPTR (gst_base_transform_acceptcaps_default);
 }
 
 static void
@@ -398,6 +427,10 @@ gst_base_transform_init (GstBaseTransform * trans,
       GST_DEBUG_FUNCPTR (gst_base_transform_sink_activate_push));
   gst_pad_set_bufferalloc_function (trans->sinkpad,
       GST_DEBUG_FUNCPTR (gst_base_transform_buffer_alloc));
+  gst_pad_set_query_function (trans->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_base_transform_query));
+  gst_pad_set_query_type_function (trans->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_base_transform_query_type));
   gst_element_add_pad (GST_ELEMENT (trans), trans->sinkpad);
 
   pad_template =
@@ -416,6 +449,10 @@ gst_base_transform_init (GstBaseTransform * trans,
       GST_DEBUG_FUNCPTR (gst_base_transform_getrange));
   gst_pad_set_activatepull_function (trans->srcpad,
       GST_DEBUG_FUNCPTR (gst_base_transform_src_activate_pull));
+  gst_pad_set_query_function (trans->srcpad,
+      GST_DEBUG_FUNCPTR (gst_base_transform_query));
+  gst_pad_set_query_type_function (trans->srcpad,
+      GST_DEBUG_FUNCPTR (gst_base_transform_query_type));
   gst_element_add_pad (GST_ELEMENT (trans), trans->srcpad);
 
   trans->transform_lock = g_mutex_new ();
@@ -440,6 +477,7 @@ gst_base_transform_init (GstBaseTransform * trans,
 
   trans->priv->processed = 0;
   trans->priv->dropped = 0;
+  trans->priv->force_alloc = TRUE;
 }
 
 /* given @caps on the src or sink pad (given by @direction)
@@ -544,6 +582,11 @@ gst_base_transform_transform_size (GstBaseTransform * trans,
     /* if there is a custom transform function, use this */
     ret = klass->transform_size (trans, direction, caps, size, othercaps,
         othersize);
+  } else if (klass->get_unit_size == NULL) {
+    /* if there is no transform_size and no unit_size, it means the
+     * element does not modify the size of a buffer */
+    *othersize = size;
+    ret = TRUE;
   } else {
     /* there is no transform_size function, we have to use the unit_size
      * functions. This method assumes there is a fixed unit_size associated with
@@ -643,7 +686,8 @@ gst_base_transform_getcaps (GstPad * pad)
     /* and filter against the template of this pad */
     templ = gst_pad_get_pad_template_caps (pad);
     GST_DEBUG_OBJECT (pad, "our template  %" GST_PTR_FORMAT, templ);
-    temp = gst_caps_intersect (caps, templ);
+    /* We keep the caps sorted like the returned caps */
+    temp = gst_caps_intersect_full (caps, templ, GST_CAPS_INTERSECT_FIRST);
     GST_DEBUG_OBJECT (pad, "intersected %" GST_PTR_FORMAT, temp);
     gst_caps_unref (caps);
     /* this is what we can do */
@@ -700,6 +744,12 @@ gst_base_transform_configure_caps (GstBaseTransform * trans, GstCaps * in,
     ret = klass->set_caps (trans, in, out);
   }
 
+  GST_OBJECT_LOCK (trans);
+  /* make sure we reevaluate how the buffer_alloc works wrt to proxy allocating
+   * the buffer. FIXME, this triggers some quite heavy codepaths that don't need
+   * to be taken.. */
+  trans->priv->suggest_pending = TRUE;
+  GST_OBJECT_UNLOCK (trans);
   trans->negotiated = ret;
 
   return ret;
@@ -880,8 +930,6 @@ gst_base_transform_find_transform (GstBaseTransform * trans, GstPad * pad,
   /* third attempt at fixation, call the fixate vmethod and
    * ultimately call the pad fixate function. */
   if (!is_fixed) {
-    GstCaps *temp;
-
     GST_DEBUG_OBJECT (trans,
         "trying to fixate %" GST_PTR_FORMAT " on pad %s:%s",
         othercaps, GST_DEBUG_PAD_NAME (otherpad));
@@ -891,9 +939,7 @@ gst_base_transform_find_transform (GstBaseTransform * trans, GstPad * pad,
 
     /* FIXME: when fixating using the vmethod, it might make sense to fixate
      * each of the caps; but Wim doesn't see a use case for that yet */
-    temp = gst_caps_copy_nth (othercaps, 0);
-    gst_caps_unref (othercaps);
-    othercaps = temp;
+    gst_caps_truncate (othercaps);
     peer_checked = FALSE;
 
     if (klass->fixate_caps) {
@@ -983,17 +1029,15 @@ error_cleanup:
 }
 
 static gboolean
-gst_base_transform_acceptcaps (GstPad * pad, GstCaps * caps)
+gst_base_transform_acceptcaps_default (GstBaseTransform * trans,
+    GstPadDirection direction, GstCaps * caps)
 {
-  GstBaseTransform *trans;
 #if 0
   GstPad *otherpad;
   GstCaps *othercaps = NULL;
 #endif
   gboolean ret = TRUE;
 
-  trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
-
 #if 0
   otherpad = (pad == trans->srcpad) ? trans->sinkpad : trans->srcpad;
 
@@ -1004,16 +1048,20 @@ gst_base_transform_acceptcaps (GstPad * pad, GstCaps * caps)
   {
     GstCaps *allowed;
 
-    GST_DEBUG_OBJECT (pad, "non fixed accept caps %" GST_PTR_FORMAT, caps);
+    GST_DEBUG_OBJECT (trans, "non fixed accept caps %" GST_PTR_FORMAT, caps);
 
     /* get all the formats we can handle on this pad */
-    allowed = gst_pad_get_caps_reffed (pad);
+    if (direction == GST_PAD_SRC)
+      allowed = gst_pad_get_caps_reffed (trans->srcpad);
+    else
+      allowed = gst_pad_get_caps_reffed (trans->sinkpad);
+
     if (!allowed) {
-      GST_DEBUG_OBJECT (pad, "gst_pad_get_caps() failed");
+      GST_DEBUG_OBJECT (trans, "gst_pad_get_caps() failed");
       goto no_transform_possible;
     }
 
-    GST_DEBUG_OBJECT (pad, "allowed caps %" GST_PTR_FORMAT, allowed);
+    GST_DEBUG_OBJECT (trans, "allowed caps %" GST_PTR_FORMAT, allowed);
 
     /* intersect with the requested format */
     ret = gst_caps_can_intersect (allowed, caps);
@@ -1042,14 +1090,13 @@ done:
   if (othercaps)
     gst_caps_unref (othercaps);
 #endif
-  gst_object_unref (trans);
 
   return ret;
 
   /* ERRORS */
 no_transform_possible:
   {
-    GST_WARNING_OBJECT (trans,
+    GST_DEBUG_OBJECT (trans,
         "transform could not transform %" GST_PTR_FORMAT
         " in anything we support", caps);
     ret = FALSE;
@@ -1057,6 +1104,24 @@ no_transform_possible:
   }
 }
 
+static gboolean
+gst_base_transform_acceptcaps (GstPad * pad, GstCaps * caps)
+{
+  gboolean ret = TRUE;
+  GstBaseTransform *trans;
+  GstBaseTransformClass *bclass;
+
+  trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
+  bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
+
+  if (bclass->accept_caps)
+    ret = bclass->accept_caps (trans, GST_PAD_DIRECTION (pad), caps);
+
+  gst_object_unref (trans);
+
+  return ret;
+}
+
 /* called when new caps arrive on the sink or source pad,
  * We try to find the best caps for the other side using our _find_transform()
  * function. If there are caps, we configure the transform for this new
@@ -1125,6 +1190,8 @@ gst_base_transform_setcaps (GstPad * pad, GstCaps * caps)
   }
 
 done:
+  /* new caps, force alloc on next buffer on the chain */
+  trans->priv->force_alloc = TRUE;
   if (otherpeer)
     gst_object_unref (otherpeer);
   if (othercaps)
@@ -1154,6 +1221,107 @@ failed_configure:
   }
 }
 
+static gboolean
+gst_base_transform_query (GstPad * pad, GstQuery * query)
+{
+  gboolean ret = FALSE;
+  GstBaseTransform *trans;
+  GstPad *otherpad;
+
+  trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
+  if (G_UNLIKELY (trans == NULL))
+    return FALSE;
+  otherpad = (pad == trans->srcpad) ? trans->sinkpad : trans->srcpad;
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_POSITION:{
+      GstFormat format;
+
+      gst_query_parse_position (query, &format, NULL);
+      if (format == GST_FORMAT_TIME && trans->segment.format == GST_FORMAT_TIME) {
+        gint64 pos;
+        ret = TRUE;
+
+        if ((pad == trans->sinkpad)
+            || (trans->priv->last_stop_out == GST_CLOCK_TIME_NONE)) {
+          pos =
+              gst_segment_to_stream_time (&trans->segment, GST_FORMAT_TIME,
+              trans->segment.last_stop);
+        } else {
+          pos = gst_segment_to_stream_time (&trans->segment, GST_FORMAT_TIME,
+              trans->priv->last_stop_out);
+        }
+        gst_query_set_position (query, format, pos);
+      } else {
+        ret = gst_pad_peer_query (otherpad, query);
+      }
+      break;
+    }
+    default:
+      ret = gst_pad_peer_query (otherpad, query);
+      break;
+  }
+
+  gst_object_unref (trans);
+  return ret;
+}
+
+static const GstQueryType *
+gst_base_transform_query_type (GstPad * pad)
+{
+  static const GstQueryType types[] = {
+    GST_QUERY_POSITION,
+    GST_QUERY_NONE
+  };
+
+  return types;
+}
+
+static void
+compute_upstream_suggestion (GstBaseTransform * trans, guint expsize,
+    GstCaps * caps)
+{
+  GstCaps *othercaps;
+  GstBaseTransformPrivate *priv = trans->priv;
+
+  GST_DEBUG_OBJECT (trans, "trying to find upstream suggestion");
+
+  /* we cannot convert the current buffer but we might be able to suggest a
+   * new format upstream, try to find what the best format is. */
+  othercaps = gst_base_transform_find_transform (trans, trans->srcpad, caps);
+
+  if (!othercaps) {
+    GST_DEBUG_OBJECT (trans, "incompatible caps, ignoring");
+    /* we received caps that we cannot transform. Upstream is behaving badly
+     * because it should have checked if we could handle these caps. We can
+     * simply ignore these caps and produce a buffer with our original caps. */
+  } else {
+    guint size_suggest;
+
+    GST_DEBUG_OBJECT (trans, "getting size of suggestion");
+
+    /* not a subset, we have a new upstream suggestion, remember it and
+     * allocate a default buffer. First we try to convert the size */
+    if (gst_base_transform_transform_size (trans,
+            GST_PAD_SRC, caps, expsize, othercaps, &size_suggest)) {
+
+      /* ok, remember the suggestions now */
+      GST_DEBUG_OBJECT (trans,
+          "storing new caps and size suggestion of %u and %" GST_PTR_FORMAT,
+          size_suggest, othercaps);
+
+      GST_OBJECT_LOCK (trans->sinkpad);
+      if (priv->sink_suggest)
+        gst_caps_unref (priv->sink_suggest);
+      priv->sink_suggest = gst_caps_ref (othercaps);
+      priv->size_suggest = size_suggest;
+      trans->priv->suggest_pending = TRUE;
+      GST_OBJECT_UNLOCK (trans->sinkpad);
+    }
+    gst_caps_unref (othercaps);
+  }
+}
+
 /* Allocate a buffer using gst_pad_alloc_buffer
  *
  * This function can do renegotiation on the source pad
@@ -1245,12 +1413,18 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
     goto alloc_failed;
 
   if (*out_buf == NULL) {
-    GST_DEBUG_OBJECT (trans, "doing alloc with caps %" GST_PTR_FORMAT, oldcaps);
+    if (trans->passthrough && !trans->priv->force_alloc) {
+      GST_DEBUG_OBJECT (trans, "Avoiding pad alloc");
+      *out_buf = gst_buffer_ref (in_buf);
+    } else {
+      GST_DEBUG_OBJECT (trans, "doing alloc with caps %" GST_PTR_FORMAT,
+          oldcaps);
 
-    ret = gst_pad_alloc_buffer (trans->srcpad,
-        GST_BUFFER_OFFSET (in_buf), outsize, oldcaps, out_buf);
-    if (ret != GST_FLOW_OK)
-      goto alloc_failed;
+      ret = gst_pad_alloc_buffer (trans->srcpad,
+          GST_BUFFER_OFFSET (in_buf), outsize, oldcaps, out_buf);
+      if (ret != GST_FLOW_OK)
+        goto alloc_failed;
+    }
   }
 
   /* must always have a buffer by now */
@@ -1260,6 +1434,7 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
   /* check if we got different caps on this new output buffer */
   newcaps = GST_BUFFER_CAPS (*out_buf);
   newsize = GST_BUFFER_SIZE (*out_buf);
+
   if (newcaps && !gst_caps_is_equal (newcaps, oldcaps)) {
     GstCaps *othercaps;
     gboolean can_convert;
@@ -1268,19 +1443,61 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
 
     incaps = GST_PAD_CAPS (trans->sinkpad);
 
+    /* check if we can convert the current incaps to the new target caps */
+    can_convert =
+        gst_base_transform_can_transform (trans, trans->sinkpad, incaps,
+        newcaps);
+
+    if (!can_convert) {
+      GST_DEBUG_OBJECT (trans, "cannot perform transform on current buffer");
+
+      gst_base_transform_transform_size (trans,
+          GST_PAD_SINK, incaps, GST_BUFFER_SIZE (in_buf), newcaps, &expsize);
+
+      compute_upstream_suggestion (trans, expsize, newcaps);
+
+      /* we got a suggested caps but we can't transform to it. See if there is
+       * another downstream format that we can transform to */
+      othercaps =
+          gst_base_transform_find_transform (trans, trans->sinkpad, incaps);
+
+      if (othercaps && !gst_caps_is_empty (othercaps)) {
+        GST_DEBUG_OBJECT (trans, "we found target caps %" GST_PTR_FORMAT,
+            othercaps);
+        *out_buf = gst_buffer_make_metadata_writable (*out_buf);
+        gst_buffer_set_caps (*out_buf, othercaps);
+        gst_caps_unref (othercaps);
+        newcaps = GST_BUFFER_CAPS (*out_buf);
+        can_convert = TRUE;
+      } else if (othercaps)
+        gst_caps_unref (othercaps);
+    }
+
     /* it's possible that the buffer we got is of the wrong size, get the
      * expected size here, we will check the size if we are going to use the
      * buffer later on. */
     gst_base_transform_transform_size (trans,
         GST_PAD_SINK, incaps, GST_BUFFER_SIZE (in_buf), newcaps, &expsize);
 
-    /* check if we can convert the current incaps to the new target caps */
-    can_convert =
-        gst_base_transform_can_transform (trans, trans->sinkpad, incaps,
-        newcaps);
-
     if (can_convert) {
       GST_DEBUG_OBJECT (trans, "reconfigure transform for current buffer");
+
+      /* subclass might want to add fields to the caps */
+      if (bclass->fixate_caps != NULL) {
+        newcaps = gst_caps_copy (newcaps);
+
+        GST_DEBUG_OBJECT (trans, "doing fixate %" GST_PTR_FORMAT
+            " using caps %" GST_PTR_FORMAT
+            " on pad %s:%s using fixate_caps vmethod", newcaps, incaps,
+            GST_DEBUG_PAD_NAME (trans->srcpad));
+        bclass->fixate_caps (trans, GST_PAD_SINK, incaps, newcaps);
+
+        *out_buf = gst_buffer_make_metadata_writable (*out_buf);
+        gst_buffer_set_caps (*out_buf, newcaps);
+        gst_caps_unref (newcaps);
+        newcaps = GST_BUFFER_CAPS (*out_buf);
+      }
+
       /* caps not empty, try to renegotiate to the new format */
       if (!gst_base_transform_configure_caps (trans, incaps, newcaps)) {
         /* not sure we need to fail hard here, we can simply continue our
@@ -1290,6 +1507,9 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
       /* new format configure, and use the new output buffer */
       gst_pad_set_caps (trans->srcpad, newcaps);
       discard = FALSE;
+      /* clear previous cached sink-pad caps, so buffer_alloc knows that
+       * it needs to revisit the decision about whether to proxy or not: */
+      gst_caps_replace (&priv->sink_alloc, NULL);
       /* if we got a buffer of the wrong size, discard it now and make sure we
        * allocate a propertly sized buffer later. */
       if (newsize != expsize) {
@@ -1299,47 +1519,18 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
       }
       outsize = expsize;
     } else {
-      GST_DEBUG_OBJECT (trans, "cannot perform transform on current buffer");
+      compute_upstream_suggestion (trans, expsize, newcaps);
 
-      /* we cannot convert the current buffer but we might be able to suggest a
-       * new format upstream, try to find what the best format is. */
-      othercaps =
-          gst_base_transform_find_transform (trans, trans->srcpad, newcaps);
-
-      if (!othercaps) {
-        GST_DEBUG_OBJECT (trans, "incompatible caps, ignoring");
-        /* we received caps that we cannot transform. Upstream is behaving badly
-         * because it should have checked if we could handle these caps. We can
-         * simply ignore these caps and produce a buffer with our original caps. */
-      } else {
-        guint size_suggest;
-
-        GST_DEBUG_OBJECT (trans, "getting size of suggestion");
-
-        /* not a subset, we have a new upstream suggestion, remember it and
-         * allocate a default buffer. First we try to convert the size */
-        if (gst_base_transform_transform_size (trans,
-                GST_PAD_SRC, newcaps, expsize, othercaps, &size_suggest)) {
-
-          /* ok, remember the suggestions now */
-          GST_DEBUG_OBJECT (trans,
-              "storing new caps and size suggestion of %u and %" GST_PTR_FORMAT,
-              size_suggest, othercaps);
-
-          GST_OBJECT_LOCK (trans->sinkpad);
-          if (priv->sink_suggest)
-            gst_caps_unref (priv->sink_suggest);
-          priv->sink_suggest = gst_caps_ref (othercaps);
-          priv->size_suggest = size_suggest;
-          trans->priv->suggest_pending = TRUE;
-          GST_OBJECT_UNLOCK (trans->sinkpad);
-        }
-        gst_caps_unref (othercaps);
-      }
       if (in_buf != *out_buf)
         gst_buffer_unref (*out_buf);
       *out_buf = NULL;
     }
+  } else if (outsize != newsize) {
+    GST_WARNING_OBJECT (trans, "Caps did not change but allocated size does "
+        "not match expected size (%d != %d)", newsize, outsize);
+    if (in_buf != *out_buf)
+      gst_buffer_unref (*out_buf);
+    *out_buf = NULL;
   }
 
   /* these are the final output caps */
@@ -1521,13 +1712,18 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
     GstCaps * caps, GstBuffer ** buf)
 {
   GstBaseTransform *trans;
+  GstBaseTransformClass *klass;
   GstBaseTransformPrivate *priv;
   GstFlowReturn res;
+  gboolean alloced = FALSE;
   gboolean proxy, suggest, same_caps;
   GstCaps *sink_suggest = NULL;
   guint size_suggest;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
+  if (G_UNLIKELY (trans == NULL))
+    return GST_FLOW_WRONG_STATE;
+  klass = GST_BASE_TRANSFORM_GET_CLASS (trans);
   priv = trans->priv;
 
   GST_DEBUG_OBJECT (pad, "alloc with caps %p %" GST_PTR_FORMAT ", size %u",
@@ -1600,6 +1796,20 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
         if (gst_caps_is_empty (sink_suggest))
           goto not_supported;
 
+        /* try the alloc caps if it is still not fixed */
+        if (!gst_caps_is_fixed (sink_suggest)) {
+          GstCaps *intersect;
+
+          GST_DEBUG_OBJECT (trans, "Checking if the input caps is compatible "
+              "with the non-fixed caps suggestion");
+          intersect = gst_caps_intersect (sink_suggest, caps);
+          if (!gst_caps_is_empty (intersect)) {
+            GST_DEBUG_OBJECT (trans, "It is, using it");
+            gst_caps_replace (&sink_suggest, caps);
+          }
+          gst_caps_unref (intersect);
+        }
+
         /* be safe and call default fixate */
         sink_suggest = gst_caps_make_writable (sink_suggest);
         gst_pad_fixate_caps (GST_BASE_TRANSFORM_SINK_PAD (trans), sink_suggest);
@@ -1609,15 +1819,53 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
           sink_suggest = NULL;
         }
 
-        GST_DEBUG_OBJECT (trans, "Caps fixated to now: %" GST_PTR_FORMAT,
+        GST_DEBUG_OBJECT (trans, "Caps fixed to: %" GST_PTR_FORMAT,
             sink_suggest);
       }
 
       if (sink_suggest) {
         templ = gst_pad_get_pad_template_caps (pad);
 
-        if (!gst_caps_can_intersect (sink_suggest, templ))
-          goto not_supported;
+        if (!gst_caps_can_intersect (sink_suggest, templ)) {
+          GstCaps *allowed;
+          GstCaps *peercaps;
+
+          GST_DEBUG_OBJECT (trans,
+              "Requested pad alloc caps are not supported: %" GST_PTR_FORMAT,
+              sink_suggest);
+          /* the requested pad alloc caps are not supported, so let's try
+           * picking something allowed between the pads (they are linked,
+           * there must be something) */
+          allowed = gst_pad_get_allowed_caps (pad);
+          if (allowed && !gst_caps_is_empty (allowed)) {
+            GST_DEBUG_OBJECT (trans,
+                "pads could agree on one of the following caps: " "%"
+                GST_PTR_FORMAT, allowed);
+            allowed = gst_caps_make_writable (allowed);
+
+            if (klass->fixate_caps) {
+              peercaps =
+                  gst_pad_get_allowed_caps (GST_BASE_TRANSFORM_SRC_PAD (trans));
+              klass->fixate_caps (trans, GST_PAD_SRC, peercaps, allowed);
+              gst_caps_unref (peercaps);
+            }
+
+            /* Fixate them to be safe if the subclass didn't do it */
+            gst_caps_truncate (allowed);
+            gst_pad_fixate_caps (pad, allowed);
+            gst_caps_replace (&sink_suggest, allowed);
+            gst_caps_unref (allowed);
+
+            suggest = TRUE;
+
+            GST_DEBUG_OBJECT (trans, "Fixated suggestion caps to %"
+                GST_PTR_FORMAT, sink_suggest);
+          } else {
+            if (allowed)
+              gst_caps_unref (allowed);
+            goto not_supported;
+          }
+        }
       }
     }
 
@@ -1659,7 +1907,8 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
   GST_OBJECT_UNLOCK (pad);
 
   proxy = priv->proxy_alloc;
-  GST_DEBUG_OBJECT (trans, "doing default alloc, proxy %d", proxy);
+  GST_DEBUG_OBJECT (trans, "doing default alloc, proxy %d, suggest %d", proxy,
+      suggest);
 
   /* we only want to proxy if we have no suggestion pending, FIXME */
   if (proxy && !suggest) {
@@ -1673,6 +1922,7 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
     res = gst_pad_alloc_buffer (trans->srcpad, offset, size, caps, buf);
     if (res != GST_FLOW_OK)
       goto alloc_failed;
+    alloced = TRUE;
 
     /* check if the caps changed */
     newcaps = GST_BUFFER_CAPS (*buf);
@@ -1691,8 +1941,11 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
         GST_OBJECT_UNLOCK (pad);
       } else {
         GST_DEBUG_OBJECT (trans, "peer did not accept new caps");
-        /* peer does not accept the caps, free the buffer we received and
-         * create a buffer of the requested format by the default handler. */
+        /* peer does not accept the caps, disable proxy_alloc, free the
+         * buffer we received and create a buffer of the requested format
+         * by the default handler. */
+        GST_DEBUG_OBJECT (trans, "disabling proxy");
+        priv->proxy_alloc = FALSE;
         gst_buffer_unref (*buf);
         *buf = NULL;
       }
@@ -1712,10 +1965,17 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
     sink_suggest = NULL;
   }
 
-  gst_object_unref (trans);
   if (sink_suggest)
     gst_caps_unref (sink_suggest);
 
+  if (res == GST_FLOW_OK && alloced) {
+    /* just alloc'ed a buffer, so we only want to do this again if we
+     * received a buffer */
+    GST_DEBUG_OBJECT (trans, "Cleaning force alloc");
+    trans->priv->force_alloc = FALSE;
+  }
+
+  gst_object_unref (trans);
   return res;
 
   /* ERRORS */
@@ -1746,6 +2006,10 @@ gst_base_transform_sink_event (GstPad * pad, GstEvent * event)
   gboolean forward = TRUE;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
+  if (G_UNLIKELY (trans == NULL)) {
+    gst_event_unref (event);
+    return FALSE;
+  }
   bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
 
   if (bclass->event)
@@ -1781,6 +2045,7 @@ gst_base_transform_sink_eventfunc (GstBaseTransform * trans, GstEvent * event)
       /* we need new segment info after the flush. */
       trans->have_newsegment = FALSE;
       gst_segment_init (&trans->segment, GST_FORMAT_UNDEFINED);
+      trans->priv->last_stop_out = GST_CLOCK_TIME_NONE;
       break;
     case GST_EVENT_EOS:
       break;
@@ -1833,10 +2098,17 @@ gst_base_transform_src_event (GstPad * pad, GstEvent * event)
   gboolean ret = TRUE;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
+  if (G_UNLIKELY (trans == NULL)) {
+    gst_event_unref (event);
+    return FALSE;
+  }
+
   bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
 
   if (bclass->src_event)
     ret = bclass->src_event (trans, event);
+  else
+    gst_event_unref (event);
 
   gst_object_unref (trans);
 
@@ -1848,6 +2120,8 @@ gst_base_transform_src_eventfunc (GstBaseTransform * trans, GstEvent * event)
 {
   gboolean ret;
 
+  GST_DEBUG_OBJECT (trans, "handling event %p %" GST_PTR_FORMAT, event, event);
+
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
       break;
@@ -2013,10 +2287,17 @@ no_qos:
       GST_DEBUG_OBJECT (trans, "doing inplace transform");
 
       if (inbuf != *outbuf) {
-        /* different buffers, copy the input to the output first, we then do an
-         * in-place transform on the output buffer. */
-        memcpy (GST_BUFFER_DATA (*outbuf), GST_BUFFER_DATA (inbuf),
-            GST_BUFFER_SIZE (inbuf));
+        guint8 *indata, *outdata;
+
+        /* Different buffer. The data can still be the same when we are dealing
+         * with subbuffers of the same buffer. Note that because of the FIXME in
+         * prepare_output_buffer() we have decreased the refcounts of inbuf and
+         * outbuf to keep them writable */
+        indata = GST_BUFFER_DATA (inbuf);
+        outdata = GST_BUFFER_DATA (*outbuf);
+
+        if (indata != outdata)
+          memcpy (outdata, indata, GST_BUFFER_SIZE (inbuf));
       }
       ret = bclass->transform_ip (trans, *outbuf);
     } else {
@@ -2034,6 +2315,9 @@ skip:
   if (*outbuf != inbuf)
     gst_buffer_unref (inbuf);
 
+  /* pushed a buffer, we can now try an alloc */
+  GST_DEBUG_OBJECT (trans, "Pushed a buffer, setting force alloc to true");
+  trans->priv->force_alloc = TRUE;
   return ret;
 
   /* ERRORS */
@@ -2144,11 +2428,24 @@ gst_base_transform_chain (GstPad * pad, GstBuffer * buffer)
    * GST_BASE_TRANSFORM_FLOW_DROPPED we will not push either. */
   if (outbuf != NULL) {
     if ((ret == GST_FLOW_OK)) {
+      GstClockTime last_stop_out = GST_CLOCK_TIME_NONE;
+
       /* Remember last stop position */
-      if ((last_stop != GST_CLOCK_TIME_NONE) &&
-          (trans->segment.format == GST_FORMAT_TIME))
+      if (last_stop != GST_CLOCK_TIME_NONE &&
+          trans->segment.format == GST_FORMAT_TIME)
         gst_segment_set_last_stop (&trans->segment, GST_FORMAT_TIME, last_stop);
 
+      if (GST_BUFFER_TIMESTAMP_IS_VALID (outbuf)) {
+        last_stop_out = GST_BUFFER_TIMESTAMP (outbuf);
+        if (GST_BUFFER_DURATION_IS_VALID (outbuf))
+          last_stop_out += GST_BUFFER_DURATION (outbuf);
+      } else if (last_stop != GST_CLOCK_TIME_NONE) {
+        last_stop_out = last_stop;
+      }
+      if (last_stop_out != GST_CLOCK_TIME_NONE
+          && trans->segment.format == GST_FORMAT_TIME)
+        trans->priv->last_stop_out = last_stop_out;
+
       /* apply DISCONT flag if the buffer is not yet marked as such */
       if (trans->priv->discont) {
         if (!GST_BUFFER_IS_DISCONT (outbuf)) {
@@ -2234,12 +2531,14 @@ gst_base_transform_activate (GstBaseTransform * trans, gboolean active)
     trans->negotiated = FALSE;
     trans->have_newsegment = FALSE;
     gst_segment_init (&trans->segment, GST_FORMAT_UNDEFINED);
+    trans->priv->last_stop_out = GST_CLOCK_TIME_NONE;
     trans->priv->proportion = 1.0;
     trans->priv->earliest_time = -1;
     trans->priv->discont = FALSE;
     gst_caps_replace (&trans->priv->sink_suggest, NULL);
     trans->priv->processed = 0;
     trans->priv->dropped = 0;
+    trans->priv->force_alloc = TRUE;
 
     GST_OBJECT_UNLOCK (trans);
   } else {
@@ -2541,7 +2840,7 @@ gst_base_transform_set_gap_aware (GstBaseTransform * trans, gboolean gap_aware)
 /**
  * gst_base_transform_suggest:
  * @trans: a #GstBaseTransform
- * @caps: caps to suggest
+ * @caps: (transfer none): caps to suggest
  * @size: buffer size to suggest
  *
  * Instructs @trans to suggest new @caps upstream. A copy of @caps will be
